using JuMP
using Gurobi
using MAT
using Printf
# using CSV
# using DataFrames
# using Plots
# using JLD

# High-level Settings
Zone = "NYC" # price zone name

# read price
fileln = matopen(string("./data/RTP_",Zone,"_2010_2019.mat"))
RTP = read(fileln, "RTP")
close(fileln)
# fileln = string("./data/RTP_",Zone,"_2010_2019.csv")
# RTP = DataFrame(CSV.File(fileln))
# RTP= RTP[:, 2:end];

#

J = 3;

E = 1;  # storage energy capacity in MWh

BatType = 1

if BatType == 0
    Ncyc = [7700 7200 6400]; # LFP
elseif BatType == 1
    Ncyc = [2200 1600 500]; # NMC
else
    Ncyc = [1400 600 350]; # NCA
end


R = 150e3;

Eseg = [0.2 0.4 0.4];

c_pu = zeros(1,3);
c_pu[1] = 5*R*0.2/Ncyc[1];
c_pu[2] = 5*R*(0.2/Ncyc[2]*sum(Eseg[1:2]) - 0.2/Ncyc[1]*Eseg[1])/Eseg[2];
c_pu[3] = 5*R*(0.2/Ncyc[3]*sum(Eseg[1:3]) - 0.2/Ncyc[2]*sum(Eseg[1:2]))/Eseg[3];

# c_pu[2] = c_pu[1]
# c_pu[3] = c_pu[1]
# c_pu = c_pu[3]
# Eseg = 1
# simulation setting
T = 288; # time step per day
M = 1/12; # duaration per step in hour
N_sim = 3650; # number of days
L = RTP[:,1];

Lp = L .> 0

# BES setting
P = .5; # power rating MW
# E = 2; # energy rating MWh
eta = .9; # single-trip efficiency
e0 = .0 * E;
ef = e0;

# initialize optimization model
model = Model(Gurobi.Optimizer)
set_silent(model) # no outputs
# total discharge power
@variable(model, p_d[1:T, 1:J], lower_bound = 0 )
# total discharge power
@variable(model, d[1:T], lower_bound = 0 )
# total charge power
@variable(model, c[1:T], lower_bound = 0 )
# total energy level
@variable(model, e[1:T])
@variable(model, D) # incremental cycle degradation
@variable(model, C) # value of battery capacity at the end of operation
@variable(model, R) # market revenue

# arbitrage revenue
@constraint(model, ArbRev, R == M*sum(L.*(d-c)) )
# degradation increments
# piece-wise linear degradation opportunity value
@constraint(model, DegCost, C == M*sum(c_pu.*sum(p_d, dims = 1) )  )
# initial SoC evolution
@constraint(model, SoCInit, e[1] - e0 == M*(c[1] - d[1]) )
# rest SoC evolution
@constraint(model, SoCCont[t = 2:T], e[t] - e[t-1] == M*(c[t]*eta - d[t]/eta) )
# final energy level
@constraint(model, Enelast, e[T] >= ef )
# max discharge power
@constraint(model, DisSum[t=1:T], d[t] == sum(p_d[t,:]))
# max discharge power
@constraint(model, DisSeg[t=1:T, j=1:J], p_d[t,j] <= P*Eseg[j])
# max discharge power
@constraint(model, DisMax[t=1:T], d[t] <= P*Lp[t] )
# max charge power
@constraint(model, ChrMax[t=1:T], c[t] <= P )
# max energy level
@constraint(model, SoCMax[t=1:T], e[t] <= E )
# warranty constraint
# @constraint(model, ETHMax, M*sum(d)/eta <= E-E*D0 )
# warranty constraint
# @constraint(model, ETHMax, M*sum(d)/eta <= E-E*D0 )

# maximize revenue plus degradation value
@objective(model, Max, R-C)



# initialize
R_s = zeros(1, N_sim)
P_s = zeros(1, N_sim)
C_s = zeros(1, N_sim)
D_s = zeros(1, N_sim)
e_S = zeros(T, N_sim)
c_S = zeros(T, N_sim)
d_S = zeros(T, N_sim)


@printf("Optimization starts...\n")
@time begin
for n = (N_sim-364):(N_sim)


# update prices
local L = RTP[:,n]
local Lp = L .> 0

# update prices in constraints
for t = 1:T
    set_normalized_coefficient(ArbRev, d[t], -M*L[t] )
    set_normalized_coefficient(ArbRev, c[t], M*L[t] )
    set_normalized_rhs(DisMax[t], P*Lp[t])
end


optimize!(model)

global R_s[n] = value(R)# objective_value(model);

global C_s[n] = value(C)
global P_s[n] = value(R)-value(C)
global D_s[n] = sum(value.(d))
global e_S[:,n] = value.(e)
global d_S[:,n] = value.(d)
global c_S[:,n] = value.(c)

# termination_status(model)
@printf("Finished Day %d, Cum Rev %d, Cum Profit %d, Cum Cost %d, OptStatus: %s \n", n, sum(R_s), sum(P_s), sum(C_s), termination_status(model))


end
end



file = matopen(string("./results_arb/",Zone,"_NMC_PW.mat"), "w")
# write(file, "D_s", collect(D_s))
write(file, "E", E)
write(file, "P", P)
write(file, "eta", eta)
write(file, "J", J)
write(file, "e", vec(e_S))
write(file, "c", vec(c_S))
write(file, "d", vec(d_S))
# write(file, "Ycal", Ycal)
# write(file, "P_recycle", P_recycle)
# write(file, "Vs0", Vs0)
# write(file, "EoL", EoL)
# write(file, "EoW", EoW)
close(file)

# save(string(pwd(), "\\results_jl\\",Zone,"2010_2019_",FN,".jld"), "Vs", Vs, "Ds", Ds, "P", P, "E", E)



# end
